@@ -25,25 +25,15 @@
 import logging
 import urllib.parse
 from html.parser import HTMLParser
-from urllib.error import HTTPError
+from io import BytesIO
 
-import podcastparser
+from requests.exceptions import RequestException
 
-from gpodder import util
+from gpodder import util, youtube
 
 logger = logging.getLogger(__name__)
 
 
-try:
-    # Python 2
-    from rfc822 import mktime_tz
-    from StringIO import StringIO
-except ImportError:
-    # Python 3
-    from email.utils import mktime_tz
-    from io import StringIO
-
-
 class ExceptionWithData(Exception):
     """Base exception with additional payload"""
     def __init__(self, data):
@@ -105,13 +95,19 @@ class FeedAutodiscovery(HTMLParser):
             attrs = dict(attrs)
 
             is_feed = attrs.get('type', '') in Fetcher.FEED_TYPES
+            is_youtube = 'youtube.com' in self._base
             is_alternate = attrs.get('rel', '') == 'alternate'
+            is_canonical = attrs.get('rel', '') == 'canonical'
             url = attrs.get('href', None)
             url = urllib.parse.urljoin(self._base, url)
 
             if is_feed and is_alternate and url:
                 logger.info('Feed autodiscovery: %s', url)
                 self._resolved_url = url
+            elif is_youtube and is_canonical and url:
+                url = youtube.parse_youtube_url(url)
+                logger.info('Feed autodiscovery: %s', url)
+                self._resolved_url = url
 
 
 class Fetcher(object):
@@ -132,105 +128,86 @@ class Fetcher(object):
         """
         return None
 
-    def _normalize_status(self, status):
-        # Based on Mark Pilgrim's "Atom aggregator behaviour" article
-        if status in (200, 301, 302, 304, 400, 401, 403, 404, 410, 500):
-            return status
-        elif status >= 200 and status < 300:
-            return 200
-        elif status >= 300 and status < 400:
-            return 302
-        elif status >= 400 and status < 500:
-            return 400
-        elif status >= 500 and status < 600:
-            return 500
-        else:
-            return status
-
-    def _check_statuscode(self, response, feed):
-        status = self._normalize_status(response.getcode())
-
-        if status == 200:
-            return Result(UPDATED_FEED, feed)
-        elif status == 301:
-            return Result(NEW_LOCATION, feed)
-        elif status == 302:
-            return Result(UPDATED_FEED, feed)
+    @staticmethod
+    def _check_statuscode(status, url):
+        if status >= 200 and status < 300:
+            return UPDATED_FEED
         elif status == 304:
-            return Result(NOT_MODIFIED, feed)
+            return NOT_MODIFIED
+        # redirects are handled by requests directly
+        # => the status should never be 301, 302, 303, 307, 308
 
-        if status == 400:
-            raise BadRequest('bad request')
-        elif status == 401:
-            raise AuthenticationRequired('authentication required', feed)
+        if status == 401:
+            raise AuthenticationRequired('authentication required', url)
         elif status == 403:
             raise Unsubscribe('forbidden')
         elif status == 404:
             raise NotFound('not found')
         elif status == 410:
             raise Unsubscribe('resource is gone')
-        elif status == 500:
+        elif status >= 400 and status < 500:
+            raise BadRequest('bad request')
+        elif status >= 500 and status < 600:
             raise InternalServerError('internal server error')
         else:
             raise UnknownStatusCode(status)
 
-    def _parse_feed(self, url, etag, modified, autodiscovery=True, max_episodes=0):
+    def parse_feed(self, url, data_stream, headers, status, **kwargs):
+        """
+        kwargs are passed from Fetcher.fetch
+        :param str url: real url
+        :param data_stream: file-like object to read from (bytes mode)
+        :param dict-like headers: response headers (may be empty)
+        :param int status: always UPDATED_FEED for now
+        :return Result: Result(status, model.Feed from parsed data_stream)
+        """
+        raise NotImplementedError("Implement parse_feed()")
+
+    def fetch(self, url, etag=None, modified=None, autodiscovery=True, **kwargs):
+        """ use kwargs to pass extra data to parse_feed in Fetcher subclasses """
+        # handle local file first
+        if url.startswith('file://'):
+            url = url[len('file://'):]
+            stream = open(url)
+            return self.parse_feed(url, stream, {}, UPDATED_FEED, **kwargs)
+
+        # remote feed
         headers = {}
         if modified is not None:
             headers['If-Modified-Since'] = modified
         if etag is not None:
             headers['If-None-Match'] = etag
 
-        if url.startswith('file://'):
-            is_local = True
-            url = url[len('file://'):]
-            stream = open(url)
-        else:
-            is_local = False
-            try:
-                stream = util.urlopen(url, headers)
-            except HTTPError as e:
-                return self._check_statuscode(e, e.geturl())
+        stream = util.urlopen(url, headers)
 
-        data = stream
-        if autodiscovery and not is_local and stream.headers.get('content-type', '').startswith('text/html'):
-            # Not very robust attempt to detect encoding: http://stackoverflow.com/a/1495675/1072626
-            charset = stream.headers.get_param('charset')
-            if charset is None:
-                charset = 'utf-8'  # utf-8 appears hard-coded elsewhere in this codebase
+        responses = stream.history + [stream]
+        for i, resp in enumerate(responses):
+            if resp.is_permanent_redirect:
+                # there should always be a next response when a redirect is encountered
+                # If max redirects is reached, TooManyRedirects is raised
+                # TODO: since we've got the end contents anyway, modify model.py to accept contents on NEW_LOCATION
+                return Result(NEW_LOCATION, responses[i + 1].url)
+        res = self._check_statuscode(stream.status_code, stream.url)
+        if res == NOT_MODIFIED:
+            return Result(NOT_MODIFIED, stream.url)
 
-            # We use StringIO in case the stream needs to be read again
-            data = StringIO(stream.read().decode(charset))
+        if autodiscovery and stream.headers.get('content-type', '').startswith('text/html'):
             ad = FeedAutodiscovery(url)
-
-            ad.feed(data.getvalue())
-            if ad._resolved_url:
+            # response_text() will assume utf-8 if no charset specified
+            ad.feed(util.response_text(stream))
+            if ad._resolved_url and ad._resolved_url != url:
                 try:
-                    self._parse_feed(ad._resolved_url, None, None, False)
+                    self.fetch(ad._resolved_url, etag=None, modified=None, autodiscovery=False, **kwargs)
                     return Result(NEW_LOCATION, ad._resolved_url)
                 except Exception as e:
                     logger.warn('Feed autodiscovery failed', exc_info=True)
 
-                # Second, try to resolve the URL
-                url = self._resolve_url(url)
-                if url:
-                    return Result(NEW_LOCATION, url)
+            # Second, try to resolve the URL
+            new_url = self._resolve_url(url)
+            if new_url and new_url != url:
+                return Result(NEW_LOCATION, new_url)
 
-            # Reset the stream so podcastparser can give it a go
-            data.seek(0)
-
-        try:
-            feed = podcastparser.parse(url, data)
-            feed['url'] = url
-        except ValueError as e:
-            raise InvalidFeed('Could not parse feed: {msg}'.format(msg=e))
-
-        if is_local:
-            feed['headers'] = {}
-            return Result(UPDATED_FEED, feed)
-        else:
-            feed['headers'] = stream.headers
-            return self._check_statuscode(stream, feed)
-
-    def fetch(self, url, etag=None, modified=None, max_episodes=0):
-        return self._parse_feed(url, etag, modified, max_episodes)
+        # xml documents specify the encoding inline so better pass encoded body.
+        # Especially since requests will use ISO-8859-1 for content-type 'text/xml'
+        # if the server doesn't specify a charset.
+        return self.parse_feed(url, BytesIO(stream.content), stream.headers, UPDATED_FEED, **kwargs)
