@@ -50,7 +50,7 @@ class VimeoError(BaseException): pass
 
 
 @registry.download_url.register
-def vimeo_real_download_url(config, episode):
+def vimeo_real_download_url(config, episode, allow_partial):
     fmt = config.vimeo.fileformat if config else None
     res = get_real_download_url(episode.url, preferred_fileformat=fmt)
     return None if res == episode.url else res
@@ -65,8 +65,7 @@ def get_real_download_url(url, preferred_fileformat=None):
     data_config_url = 'https://player.vimeo.com/video/%s/config' % (video_id,)
 
     def get_urls(data_config_url):
-        data_config_data = util.urlopen(data_config_url).read().decode('utf-8')
-        data_config = json.loads(data_config_data)
+        data_config = util.urlopen(data_config_url).json()
         for fileinfo in list(data_config['request']['files'].values()):
             if not isinstance(fileinfo, list):
                 continue
