@@ -20,6 +20,7 @@
 #  Justin Forest <justin.forest@gmail.com> 2008-10-13
 #
 
+import io
 import json
 import logging
 import re
@@ -186,6 +187,39 @@ def youtube_real_download_url(config, episode, allow_partial):
     return None if res == episode.url else res
 
 
+def youtube_get_old_endpoint(vid):
+    # TODO: changing 'detailpage' to 'embedded' allows age-restricted content
+    url = 'https://www.youtube.com/get_video_info?html5=1&c=TVHTML5&cver=6.20180913&el=detailpage&video_id=' + vid
+    r = util.urlopen(url)
+    if not r.ok:
+        raise YouTubeError('Youtube "%s": %d %s' % (url, r.status_code, r.reason))
+    else:
+        return r.text, None
+
+
+def youtube_get_new_endpoint(vid):
+    url = 'https://www.youtube.com/watch?bpctr=9999999999&has_verified=1&v=' + vid
+    r = util.urlopen(url)
+    if not r.ok:
+        raise YouTubeError('Youtube "%s": %d %s' % (url, r.status_code, r.reason))
+
+    ipr = re.search(r'ytInitialPlayerResponse\s*=\s*({.+?})\s*;', r.text)
+    if ipr is None:
+        try:
+            url = get_gdpr_consent_url(r.text)
+        except YouTubeError as e:
+            raise YouTubeError('Youtube "%s": No ytInitialPlayerResponse found and %s' % (url, str(e)))
+        r = util.urlopen(url)
+        if not r.ok:
+            raise YouTubeError('Youtube "%s": %d %s' % (url, r.status_code, r.reason))
+
+        ipr = re.search(r'ytInitialPlayerResponse\s*=\s*({.+?})\s*;', r.text)
+        if ipr is None:
+            raise YouTubeError('Youtube "%s": No ytInitialPlayerResponse found' % url)
+
+    return None, ipr.group(1)
+
+
 def get_real_download_url(url, allow_partial, preferred_fmt_ids=None):
     if not preferred_fmt_ids:
         preferred_fmt_ids, _, _ = formats_dict[22]  # MP4 720p
@@ -194,31 +228,25 @@ def get_real_download_url(url, allow_partial, preferred_fmt_ids=None):
 
     vid = get_youtube_id(url)
     if vid is not None:
-        page = None
-        url = 'https://www.youtube.com/get_video_info?&el=detailpage&video_id=' + vid
+        try:
+            old_page, new_page = youtube_get_new_endpoint(vid)
+        except YouTubeError as e:
+            logger.info(str(e))
+            old_page, new_page = youtube_get_old_endpoint(vid)
 
-        while page is None:
-            req = util.http_request(url, method='GET')
-            if 'location' in req.msg:
-                url = req.msg['location']
-            else:
-                page = req.read()
-
-        page = page.decode()
-        # Try to find the best video format available for this video
-        # (http://forum.videohelp.com/topic336882-1800.html#1912972)
-
-        def find_urls(page):
+        def find_urls(old_page, new_page):
             # streamingData is preferable to url_encoded_fmt_stream_map
             # streamingData.formats are the same as url_encoded_fmt_stream_map
             # streamingData.adaptiveFormats are audio-only and video-only formats
-            x = parse_qs(page)
+
+            x = parse_qs(old_page) if old_page else json.loads(new_page)
+            player_response = json.loads(x['player_response'][0]) if old_page and 'player_response' in x else x
             error_message = None
 
             if 'reason' in x:
+                # TODO: unknown if this is valid for new_page
                 error_message = util.remove_html_tags(x['reason'][0])
-            elif 'player_response' in x:
-                player_response = json.loads(x['player_response'][0])
+            elif 'playabilityStatus' in player_response:
                 playabilityStatus = player_response['playabilityStatus']
 
                 if 'reason' in playabilityStatus:
@@ -229,18 +257,13 @@ def get_real_download_url(url, allow_partial, preferred_fmt_ids=None):
                     # playabilityStatus.liveStreamability.liveStreamabilityRenderer.displayEndscreen -- video has ended if present
 
                     if allow_partial and 'streamingData' in player_response and 'hlsManifestUrl' in player_response['streamingData']:
-                        manifest = None
-                        url = player_response['streamingData']['hlsManifestUrl']
-                        while manifest is None:
-                            req = util.http_request(url, method='GET')
-                            if 'location' in req.msg:
-                                url = req.msg['location']
-                            else:
-                                manifest = req.read()
-                        manifest = manifest.decode().splitlines()
+                        r = util.urlopen(player_response['streamingData']['hlsManifestUrl'])
+                        if not r.ok:
+                            raise YouTubeError('HLS Manifest: %d %s' % (r.status_code, r.reason))
+                        manifest = r.text.splitlines()
 
                         urls = [line for line in manifest if line[0] != '#']
-                        itag_re = re.compile('/itag/([0-9]+)/')
+                        itag_re = re.compile(r'/itag/([0-9]+)/')
                         for url in urls:
                             itag = itag_re.search(url).group(1)
                             yield int(itag), [url, None]
@@ -248,41 +271,41 @@ def get_real_download_url(url, allow_partial, preferred_fmt_ids=None):
 
                     error_message = 'live stream'
                 elif 'streamingData' in player_response:
-                    # DRM videos store url inside a cipher key - not supported
                     if 'formats' in player_response['streamingData']:
                         for f in player_response['streamingData']['formats']:
-                            if 'url' in f:
+                            if 'url' in f:  # DRM videos store url inside a signatureCipher key
                                 yield int(f['itag']), [f['url'], f.get('approxDurationMs')]
                     if 'adaptiveFormats' in player_response['streamingData']:
                         for f in player_response['streamingData']['adaptiveFormats']:
-                            if 'url' in f:
+                            if 'url' in f:  # DRM videos store url inside a signatureCipher key
                                 yield int(f['itag']), [f['url'], f.get('approxDurationMs')]
                     return
 
             if error_message is not None:
-                raise YouTubeError('Cannot download video: %s' % error_message)
+                raise YouTubeError(('Cannot stream video: %s' if allow_partial else 'Cannot download video: %s') % error_message)
 
-            r4 = re.search('url_encoded_fmt_stream_map=([^&]+)', page)
-            if r4 is not None:
-                fmt_url_map = urllib.parse.unquote(r4.group(1))
-                for fmt_url_encoded in fmt_url_map.split(','):
-                    video_info = parse_qs(fmt_url_encoded)
-                    yield int(video_info['itag'][0]), [video_info['url'][0], None]
+            if old_page:
+                r4 = re.search(r'url_encoded_fmt_stream_map=([^&]+)', old_page)
+                if r4 is not None:
+                    fmt_url_map = urllib.parse.unquote(r4.group(1))
+                    for fmt_url_encoded in fmt_url_map.split(','):
+                        video_info = parse_qs(fmt_url_encoded)
+                        yield int(video_info['itag'][0]), [video_info['url'][0], None]
 
-        fmt_id_url_map = sorted(find_urls(page), reverse=True)
+        fmt_id_url_map = sorted(find_urls(old_page, new_page), reverse=True)
 
         if not fmt_id_url_map:
-            drm = re.search('%22(cipher|signatureCipher)%22%3A', page)
+            drm = re.search(r'(%22(cipher|signatureCipher)%22%3A|"signatureCipher":)', old_page or new_page)
             if drm is not None:
-                raise YouTubeError('Unsupported DRM content found for video ID "%s"' % vid)
-            raise YouTubeError('No formats found for video ID "%s"' % vid)
+                raise YouTubeError('Unsupported DRM content')
+            raise YouTubeError('No formats found')
 
         formats_available = set(fmt_id for fmt_id, url in fmt_id_url_map)
         fmt_id_url_map = dict(fmt_id_url_map)
 
         for id in preferred_fmt_ids:
-            if re.search('\+', str(id)):
-                # skip formats that contain a + (136+140)
+            if not re.search(r'^[0-9]+$', str(id)):
+                # skip non-integer formats 'best', '136+140' or twitch '720p'
                 continue
             id = int(id)
             if id in formats_available:
@@ -297,21 +320,21 @@ def get_real_download_url(url, allow_partial, preferred_fmt_ids=None):
                 url, duration = fmt_id_url_map[id]
                 break
         else:
-            raise YouTubeError('No preferred formats found for video ID "%s"' % vid)
+            raise YouTubeError('No preferred formats found')
 
     return url, duration
 
 
 def get_youtube_id(url):
-    r = re.compile('http[s]?://(?:[a-z]+\.)?youtube\.com/v/(.*)\.swf', re.IGNORECASE).match(url)
+    r = re.compile(r'http[s]?://(?:[a-z]+\.)?youtube\.com/v/(.*)\.swf', re.IGNORECASE).match(url)
     if r is not None:
         return r.group(1)
 
-    r = re.compile('http[s]?://(?:[a-z]+\.)?youtube\.com/watch\?v=([^&]*)', re.IGNORECASE).match(url)
+    r = re.compile(r'http[s]?://(?:[a-z]+\.)?youtube\.com/watch\?v=([^&]*)', re.IGNORECASE).match(url)
     if r is not None:
         return r.group(1)
 
-    r = re.compile('http[s]?://(?:[a-z]+\.)?youtube\.com/v/(.*)[?]', re.IGNORECASE).match(url)
+    r = re.compile(r'http[s]?://(?:[a-z]+\.)?youtube\.com/v/(.*)[?]', re.IGNORECASE).match(url)
     if r is not None:
         return r.group(1)
 
@@ -334,13 +357,13 @@ def for_each_feed_pattern(func, url, fallback_result):
     func() returns None, return fallback_result.
     """
     CHANNEL_MATCH_PATTERNS = [
-        'http[s]?://(?:[a-z]+\.)?youtube\.com/user/([a-z0-9]+)',
-        'http[s]?://(?:[a-z]+\.)?youtube\.com/profile?user=([a-z0-9]+)',
-        'http[s]?://(?:[a-z]+\.)?youtube\.com/rss/user/([a-z0-9]+)/videos\.rss',
-        'http[s]?://(?:[a-z]+\.)?youtube\.com/channel/([-_a-z0-9]+)',
-        'http[s]?://(?:[a-z]+\.)?youtube\.com/feeds/videos.xml\?channel_id=([-_a-z0-9]+)',
-        'http[s]?://gdata.youtube.com/feeds/users/([^/]+)/uploads',
-        'http[s]?://gdata.youtube.com/feeds/base/users/([^/]+)/uploads',
+        r'http[s]?://(?:[a-z]+\.)?youtube\.com/user/([a-z0-9]+)',
+        r'http[s]?://(?:[a-z]+\.)?youtube\.com/profile?user=([a-z0-9]+)',
+        r'http[s]?://(?:[a-z]+\.)?youtube\.com/rss/user/([a-z0-9]+)/videos\.rss',
+        r'http[s]?://(?:[a-z]+\.)?youtube\.com/channel/([-_a-z0-9]+)',
+        r'http[s]?://(?:[a-z]+\.)?youtube\.com/feeds/videos.xml\?channel_id=([-_a-z0-9]+)',
+        r'http[s]?://gdata.youtube.com/feeds/users/([^/]+)/uploads',
+        r'http[s]?://gdata.youtube.com/feeds/base/users/([^/]+)/uploads',
     ]
 
     for pattern in CHANNEL_MATCH_PATTERNS:
@@ -365,10 +388,15 @@ def get_real_channel_url(url):
 def get_channel_id_url(url):
     if 'youtube.com' in url:
         try:
-            channel_url = ''
-            raw_xml_data = util.urlopen(url).read().decode('utf-8')
-            xml_data = xml.etree.ElementTree.fromstring(raw_xml_data)
-            channel_id = xml_data.find("{http://www.youtube.com/xml/schemas/2015}channelId").text
+            req = util.urlopen(url)
+            # video page may contain corrupt HTML/XML, search for tag to avoid exception
+            m = re.search(r'<meta itemprop="channelId" content="([^"]+)">', req.text)
+            if m:
+                channel_id = m.group(1)
+            else:
+                raw_xml_data = io.BytesIO(req.content)
+                xml_data = xml.etree.ElementTree.parse(raw_xml_data)
+                channel_id = xml_data.find("{http://www.youtube.com/xml/schemas/2015}channelId").text
             channel_url = 'https://www.youtube.com/channel/{}'.format(channel_id)
             return channel_url
 
@@ -402,7 +430,7 @@ def get_cover(url):
 
         try:
             channel_url = get_channel_id_url(url)
-            html_data = util.urlopen(channel_url).read().decode('utf-8')
+            html_data = util.response_text(util.urlopen(channel_url))
             parser = YouTubeHTMLCoverParser()
             parser.feed(html_data)
             if parser.url:
@@ -413,6 +441,46 @@ def get_cover(url):
             logger.warning('Could not retrieve cover art', exc_info=True)
 
 
+def get_gdpr_consent_url(html_data):
+    """
+    Creates the URL for automatically accepting GDPR consents
+    EU GDPR redirects to a form that needs to be posted to be redirected to a get request
+    with the form data as input to the youtube video URL. This extracts that form data from
+    the GDPR form and builds up the URL the posted form results.
+    """
+    class ConsentHTML(HTMLParser):
+        def __init__(self):
+            super().__init__()
+            self.url = ''
+            self.consentForm = False
+
+        def handle_starttag(self, tag, attributes):
+            attribute_dict = {attribute[0]: attribute[1] for attribute in attributes}
+            if tag == 'form' and attribute_dict['action'] == 'https://consent.youtube.com/s':
+                self.consentForm = True
+                self.url = 'https://consent.google.com/s?'
+            # Get GDPR form elements
+            if self.consentForm and tag == 'input' and attribute_dict['type'] == 'hidden':
+                self.url += '&' + attribute_dict['name'] + '=' + urllib.parse.quote_plus(attribute_dict['value'])
+
+        def handle_endtag(self, tag):
+            if tag == 'form':
+                self.consentForm = False
+
+    try:
+        parser = ConsentHTML()
+        parser.feed(html_data)
+    except Exception:
+        raise YouTubeError('Could not retrieve GDPR accepted consent URL')
+
+    if parser.url:
+        logger.debug('YouTube GDPR accept consent URL is: %s', parser.url)
+        return parser.url
+    else:
+        logger.debug('YouTube GDPR accepted consent URL could not be resolved.', parser.url)
+        raise YouTubeError('No acceptable GDPR consent URL')
+
+
 def get_channel_desc(url):
     if 'youtube.com' in url:
 
@@ -433,7 +501,7 @@ def get_channel_desc(url):
 
         try:
             channel_url = get_channel_id_url(url)
-            html_data = util.urlopen(channel_url).read().decode('utf-8')
+            html_data = util.response_text(util.urlopen(channel_url))
             parser = YouTubeHTMLDesc()
             parser.feed(html_data)
             if parser.description:
@@ -472,27 +540,37 @@ def parse_youtube_url(url):
     scheme, netloc, path, query, fragment = urllib.parse.urlsplit(url)
     logger.debug("Analyzing URL: {}".format(" ".join([scheme, netloc, path, query, fragment])))
 
-    if 'youtube.com' in netloc and ('/user/' in path or '/channel/' in path or 'list=' in query):
-        logger.debug("Valid Youtube URL detected. Parsing...")
+    if 'youtube.com' in netloc:
+        if path == '/feeds/videos.xml' and re.search(r'^(user|channel|playlist)_id=.*', query):
+            return url
 
-        if path.startswith('/user/'):
-            user_id = path.split('/')[2]
-            query = 'user={user_id}'.format(user_id=user_id)
+        if '/user/' in path or '/channel/' in path or 'list=' in query:
+            logger.debug("Valid Youtube URL detected. Parsing...")
 
-        if path.startswith('/channel/'):
-            channel_id = path.split('/')[2]
-            query = 'channel_id={channel_id}'.format(channel_id=channel_id)
+            if path.startswith('/user/'):
+                user_id = path.split('/')[2]
+                query = 'user={user_id}'.format(user_id=user_id)
 
-        if 'list=' in query:
-            playlist_query = [query_value for query_value in query.split("&") if 'list=' in query_value][0]
-            playlist_id = playlist_query[5:]
-            query = 'playlist_id={playlist_id}'.format(playlist_id=playlist_id)
+            if path.startswith('/channel/'):
+                channel_id = path.split('/')[2]
+                query = 'channel_id={channel_id}'.format(channel_id=channel_id)
 
-        path = '/feeds/videos.xml'
+            if 'list=' in query:
+                playlist_query = [query_value for query_value in query.split("&") if 'list=' in query_value][0]
+                playlist_id = playlist_query[5:]
+                query = 'playlist_id={playlist_id}'.format(playlist_id=playlist_id)
 
-        new_url = urllib.parse.urlunsplit((scheme, netloc, path, query, fragment))
-        logger.debug("New Youtube URL: {}".format(new_url))
-        return new_url
-    else:
-        logger.debug("Not a valid Youtube URL: {}".format(url))
-        return url
+            path = '/feeds/videos.xml'
+
+            new_url = urllib.parse.urlunsplit((scheme, netloc, path, query, fragment))
+            logger.debug("New Youtube URL: {}".format(new_url))
+            return new_url
+
+        # look for channel URL in page
+        new_url = get_channel_id_url(url)
+        if new_url:
+            logger.debug("New Youtube URL: {}".format(new_url))
+            return new_url
+
+    logger.debug("Not a valid Youtube URL: {}".format(url))
+    return url
